package it.dsestili.jhash.core;

/*
JHash a simple hash code generator
Copyright (C) 2013 Davide Sestili

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

import it.dsestili.jhash.gui.HashListWindow;
import it.dsestili.jhash.gui.InputPanel;
import it.dsestili.jhash.gui.StatusPanel;
import java.awt.Frame;
import java.io.File;
import java.io.FilenameFilter;
import java.util.Vector;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

public class DirectoryWorkerThread extends WorkerThread
{
	private volatile File currentFile;
	private volatile int currentIndex, totFiles;
	private final Frame frame;
	
	public DirectoryWorkerThread(Frame frame, File file, String algorithm, StatusPanel statusPanel, InputPanel inputPanel, JButton btnStart)
	{
		super(file, algorithm, statusPanel, inputPanel, btnStart);
		
		this.frame = frame;
	}

	@Override
	public void run()
	{
		try
		{
			manageGUI(false);
			
			File[] files = file.listFiles(new FilenameFilter() {
				
				@Override
				public boolean accept(File dir, String name) 
				{
					return new File(dir, name).isFile();
				}
				
			});

			if(files.length == 0)
			{
				JOptionPane.showMessageDialog(frame, "Empty folder!", "Empty folder", JOptionPane.INFORMATION_MESSAGE);
				return;
			}
			
			Vector<Vector<String>> hashList = new Vector<Vector<String>>();
			
			totFiles = files.length;
			
			for(int i = 0; i < files.length; i++)
			{
				currentIndex = i;
				currentFile = files[i];
				
				core = new Core(currentFile, algorithm);
				core.addIProgressListener(this);

				String hash = core.generateHash();
				
				Vector<String> info = new Vector<String>();
				info.add(currentFile.getName());
				info.add(hash);
				
				hashList.add(info);
			}

			showHash(hashList);
		}
		catch(InterruptedException e)
		{
			showStatus("Operation canceled");
		}
		catch(Throwable t)
		{
			showStatus("Error occurred");
			
			t.printStackTrace();
		}
		finally
		{
			manageGUI(true);
		}
	}

	private void showHash(final Vector<Vector<String>> hashList)
	{
		SwingUtilities.invokeLater(new Runnable() 
		{
			@Override
			public void run() 
			{
				new HashListWindow(frame, hashList);
			}
		});
	}

	@Override
	public void progressEvent(final ProgressEvent event) 
	{
		SwingUtilities.invokeLater(new Runnable() 
		{
			@Override
			public void run() 
			{
				statusPanel.setStatus("File " + (currentIndex + 1) + " of " + totFiles + ": " + currentFile.getName() + " - " + event.toString());
				statusPanel.setValue(event.getPCent());
			}
		});
	}
}
